//
//  CommunicationProtocol.hpp
//  24U Phidgets Plug-In
//
//  Created by Patrik Horcicak on 23/09/2019.
//  Copyright © 2019 Filemaker, Inc. All rights reserved.
//

#ifndef CommunicationProtocol_h
#define CommunicationProtocol_h

#include "Manager.h"

#define STR_SEPARATOR       "\n"

#include <vector>

#ifdef _WIN32
#include <mutex>
#endif // _WIN32

#include "protocolEnums.h"

class PhidgetHandlers;

/*enum PLUGIN_FUNC
{
    PLUGIN_READ             = 0x01,
    PLUGIN_WRITE            = 0x02,
    PLUGIN_SET              = 0x03,
    PLUGIN_GET              = 0x04,
    PLUGIN_CONNECT          = 0x05,
    PLUGIN_DISCONNECT       = 0x06,
    PLUGIN_SCRIPT_EVENT     = 0x07,
    PLUGIN_SERVER_EVENT       = 0x08,
};

enum PLUGIN_FUNC_SELECTOR
{
    RD_INPUT_VALUE                  = 0x01,
    RD_INPUT_STATE                  = 0x02,
    RD_SENSOR_VALUE                 = 0x03,
    RD_SENSOR_RAW_VALUE             = 0x04,
    RD_OUTPUT_STATE                 = 0x05,
    RD_TAG                          = 0x06,
    RD_MOTOR_POSITION               = 0x07,
    RD_BRIDGE_VALUE                 = 0x08,
    RD_VOLTAGE                      = 0x09,
    RD_PH                           = 0x0A,
    RD_TEMPERATURE                  = 0x0B,
    RD_ILLUMINANCE                  = 0x0C,
                
    WR_OUTPUT_STATE                 = 0x0D,
    WR_TAG                          = 0x0E,
    WR_MOTOR_POSITION               = 0x0F,
    WR_DISPLAY_STRING               = 0x10,
    WR_VOLTAGE                      = 0x11,
                
    SET_RADIOMETRIC                 = 0x11,
    SET_ANTENNA_ON                  = 0x12,
    SET_LED_ON                      = 0x13,
    SET_BACKLIGHT                   = 0x14,
    SET_BRIGHTNESS                  = 0x15,
    SET_CONTRAST                    = 0x16,
    SET_CURSOR_ON                   = 0x17,
    SET_CURSOR_BLINK                = 0x18,
    SET_ENABLED                     = 0x19,
    SET_GAIN                        = 0x1A,
    SET_DATA_RATE                   = 0x1B,
    SET_ENGAGED                     = 0x1C,
    SET_MOTOR_ON                    = 0x1D,
    SET_SERVO_MAX                   = 0x1E,
    SET_SERVO_MIN                   = 0x1F,
    SET_MIN_ACCELERATION            = 0x20,
    SET_MAX_ACCELERATION            = 0x21,
    SET_CONTROL_MODE                = 0x22,
    SET_ACCELERATION                = 0x23,
    SET_VELOCITY                    = 0x24,
    SET_CURRENT_LIMIT               = 0x25,
    SET_CORRECTION_TEMPERATURE      = 0x26,
    SET_CHANGE_PH_TRIGGER           = 0x27,
    SET_CHANGE_TEMPERATURE_TRIGGER  = 0x28,
    SET_CHANGE_ILLUMINANCE_TRIGGER  = 0x29,
    SET_PORT_MODE                   = 0x2A,
    
    GET_ANTHENNA_ON                 = 0x2B,
    GET_LED_ON                      = 0x2C,
    GET_TAG_PROTOCOL                = 0x2D,
    GET_BACKLIGHT                   = 0x2E,
    GET_BRIGHTNESS                  = 0x2F,
    GET_CONTRAST                    = 0x30,
    GET_NUM_ROWS                    = 0x31,
    GET_NUM_COLOMNS                 = 0x32,
    GET_CURSOR_ON                   = 0x33,
    GET_CURSOR_BLINK                = 0x34,
    GET_ENABLED                     = 0x35,
    GET_DATA_RATE_MIN               = 0x36,
    GET_DATA_RATE_MAX               = 0x37,
    GET_DATA_RATE                   = 0x38,
    GET_GAIN                        = 0x39,
    GET_BRIDGE_MIN                  = 0x3A,
    GET_BRIDGE_MAX                  = 0x3B,
    GET_VOLTAGE_MIN                 = 0x3C,
    GET_VOLTAGE_MAX                 = 0x3D,
    GET_SERVO_MIN                   = 0x3E,
    GET_SERVO_MAX                   = 0x3F,
    GET_ENGAGED                     = 0x40,
    GET_ACCELERATION                = 0x41,
    GET_VELOCITY                    = 0x42,
    GET_MIN_ACCELERATION            = 0x43,
    GET_MAX_ACCELERATION            = 0x44,
    GET_CONTROL_MODE                = 0x45,
    GET_TARGER_POSITION             = 0x46,
    GET_CURRENT_LIMIT               = 0x47,
    GET_CORRECTION_TEMPERATURE      = 0x48,
    GET_MIN_PH                      = 0x49,
    GET_MAX_PH                      = 0x4A,
    GET_CHANGE_PH_TRIGGER           = 0x4B,
    GET_MIN_TEMPERATURE             = 0x4C,
    GET_MAX_TEMPERATURE             = 0x4D,
    GET_CHANGE_TEMPERATURE_TRIGGER  = 0x4E,
    GET_MIN_IILUMINANACE            = 0x4F,
    GET_MAX_IILUMINANACE            = 0x50,
    GET_CHANGE_ILLUMINANCE_TRIGGER  = 0x51,
    GET_PORT_MODE                   = 0x52,
    
    SE_ON_ATTACH                    = 0x53,
    SE_ON_DETTACH                   = 0x54,
    SE_ON_INPUT_CHANGE              = 0x55,
    SE_ON_SENSOR_CHANGE             = 0x56,
    SE_ON_OUTPUT_CHANGE             = 0x57,
    SE_ON_TAG                       = 0x58,
    SE_ON_TAG_LOST                  = 0x59,
    SE_MOTOR_POS_CHANGE             = 0x5A,
    SE_ON_BRIDGE_DATA               = 0x5B,
    SE_ON_STEPPER_STOP              = 0x5C,
    
    SET_SENSOR_VALUE_CHANGE_TRIGGER = 0x5D,
    GET_SENSOR_VALUE_CHANGE_TRIGGER = 0x5E,
    SET_VOLTAGE_CHANGE_TRIGGER      = 0x5F,
    GET_VOLTAGE_CHANGE_TRIGGER      = 0x60,
    
    GET_ATTACHED_PHIDGETS           = 0x61,
    SET_BT_NAME                     = 0x62,
    SET_SCREEN_SIZE                 = 0x63,
    SET_ACTIVE_SCREEN               = 0x64,
    SE_TIME_EVENT                   = 0x65,
    RD_HUMIDITY                     = 0x66,
    GET_MIN_HUMIDITY                = 0x67,
    GET_MAX_HUMIDITY                = 0x68,
    SET_CHANGE_HUMIDITY_TRIGGER     = 0x69,
    GET_CHANGE_HUMIDITY_TRIGGER     = 0x70,
    RD_DB                           = 0x71,
    RD_DBA                          = 0x72,
    RD_DBC                          = 0x73,
    RD_OCTAVES                      = 0x74,
    GET_DB_MAX                      = 0x75,
    SET_SPL_CHANGE_TRIGGER          = 0x76,
    GET_SPL_CHANGE_TRIGGER          = 0x77,
    RD_RESISTANCE                   = 0x78,
    GET_MIN_RESISTANCE              = 0x79,
    GET_MAX_RESISTANCE              = 0x80,
    SET_CHANGE_RESISTANCE_TRIGGER   = 0x81,
    GET_CHANGE_RESISTANCE_TRIGGER   = 0x82,
    RD_ACCELERATION                 = 0x83,
    SET_CHANGE_ACCELERATION_TRIGGER = 0x84,
    GET_CHANGE_ACCELERATION_TRIGGER = 0x85,
    RD_ANGULAR_RATE                 = 0x86,
    GET_MAX_ANGULAR_RATE            = 0x87,
    GET_MIN_ANGULAR_RATE            = 0x88,
    RD_MAGNETIC_FIELD               = 0x89,
    GET_MIN_MAGNETIC_FIELD          = 0x90,
    GET_MAX_MAGNETIC_FIELD          = 0x91,
    SET_CHANGE_MAGNETIC_FIELD_TRIGGER   = 0x92,
    GET_CHANGE_MAGNETIC_FIELD_TRIGGER   = 0x93,
    RD_CURRENT                      = 0x94,
    GET_MIN_CURRENT                 = 0x95,
    GET_MAX_CURRENT                 = 0x96,
    GET_POWER_SUPPLY                = 0x97,
    SET_POWER_SUPPLY                = 0x98,
    SET_CHANGE_CURRENT_TRIGGER      = 0x99,
    GET_CHANGE_CURRENT_TRIGGER      = 0xA0,
    RD_TOUCH_VALUE                  = 0xA1,
    GET_MIN_TOUCH_VALUE             = 0xA2,
    GET_MAX_TOUCH_VALUE             = 0xA3,
    SET_CHANGE_TOUCH_VALUE_TRIGGER  = 0xA4,
    GET_CHANGE_TOUCH_VALUE_TRIGGER  = 0xA5,
    GET_IS_TOUCHED                  = 0xA6,
    GET_SENSITIVITY                 = 0xA7,
    SET_SENSITIVITY                 = 0xA8,
    SE_REMOVE                       = 0xA9,
    SE_UNKNOWN                      = 0xB0
};*/

class CommunicationProtocol
{
    public:
    
		#ifdef _WIN32
		std::mutex receive_mutex;

		#endif // _WIN32
        static CommunicationProtocol& sharedInstance();
    
        static std::vector<std::vector<std::string > > phidgetStatesFromVector(std::vector<std::string> phidgets);
    
        std::string prepareData(const PhidgetHandlers* device, PLUGIN_FUNC function, PLUGIN_FUNC_SELECTOR selector, std::vector<std::string>params);
    
        void messageReceived(std::string deviceName, std::vector<std::string> message);
    
    private:
    
        CommunicationProtocol();
        ~CommunicationProtocol();
    
        void sendData();
};

#endif /* CommunicationProtocol_h */
