//
//  BluetoothManager.cpp
//  24U Phidgets Plug-In
//
//  Created by Patrik Horcicak on 23/09/2019.
//  Copyright © 2019 Filemaker, Inc. All rights reserved.
//

#include "ConnectionManager.h"
#include "SuppFunc.h"
#include "CommunicationProtocol.h"
#ifndef  IOS_VERSION
#include "TCPClient.h"
#endif

#ifdef __APPLE__
#include "ObjectiveCWrapper.h"
#endif

ConnectionManager::ConnectionManager(std::string devName, ConnectionType conType)
{
    deviceName     = devName;
    connectionType = conType;
    
	if (connectionType == ConnectionType::BT)
	{
        #ifdef __APPLE__
        NSString * name = [NSString stringWithCString:deviceName.c_str() encoding:[NSString defaultCStringEncoding]];
        manager = [[AppleBluetoothManager alloc] initWithDeviceName: name];
        
        #else
		manager = new WindowsBluetoothManager(devName);
        #endif

	}   else
    {
        //TODO: TCP
    }
}

ConnectionManager::~ConnectionManager()
{
    this->disconnect();
}

bool ConnectionManager::connect()
{
    if (connectionType == ConnectionType::BT)
    {
        #ifdef __APPLE__
        return [manager connect];
        #else
        //TODO: Other platforms
		return manager->ConnectBLEDevice();
        #endif
    }
    else
    {
        //TODO: TCP
    }
    
    return false;
}

void ConnectionManager::disconnect()
{
    if (connectionType == ConnectionType::BT)
    {
        #ifdef __APPLE__
        return [manager disconnect];
        #else
		manager->DisconectBLEDevice();
        #endif
    }
    else
    {
        //TODO: TCP - not necessary
    }
}

std::vector<std::string> ConnectionManager::sendMessageAndWaitResponse(std::string message, bool dontSplit)
{
    Manager::TCP_mutex.lock();
    try
    {
        if (connectionType == ConnectionType::BT)
        {
            #ifdef __APPLE__
            NSString * msg = [NSString stringWithCString:message.c_str() encoding:[NSString defaultCStringEncoding]];
            
            NSString * response = [manager sendMessageAndWaitForResponse:msg];
            Manager::TCP_mutex.unlock();
            
            if (response == NULL)
            {
                return {};
            }
            
            return split([response UTF8String], "\n");

            #else

            return manager->SendMessageToBLE(message);

            #endif
        }
        else
        {
    #ifndef IOS_VERSION
            TCPClient       tcpClient;
            std::string     response;
            
            if (tcpClient.send(message, response) != 0)
            {
                Manager::TCP_mutex.unlock();
                return {};
            }
            
            if(dontSplit)
            {
                Manager::TCP_mutex.unlock();
                return {response};
            }
            
            Manager::TCP_mutex.unlock();
            return split(response, "\n");
    #endif
            
        }
    }
    catch(...)
    {
        
    }
    Manager::TCP_mutex.unlock();
    return std::vector<std::string>{};
}

void ConnectionManager::messageReceived(std::string message)
{
    CommunicationProtocol::sharedInstance().messageReceived(deviceName, split(message, "\n"));
}

ConnectionManager* newTCPConnection()
{
    ConnectionManager*      tcpManager (new ConnectionManager("localhost", ConnectionType::TCP));
    std::vector<std::string>                response    = {};
    
    Manager::GetManager().connectedDevices["localhost"]     = tcpManager;
    
    std::string message     = CommunicationProtocol::sharedInstance().prepareData(nullptr, PLUGIN_FUNC::PLUGIN_GET, PLUGIN_FUNC_SELECTOR::GET_ATTACHED_PHIDGETS, {});
    
    
    try
    {
        response = tcpManager->sendMessageAndWaitResponse(message, true);
    }
    catch(...)
    {
        tcpManager->connected = false;
        return tcpManager;
    }
    
    
    try
    {
        if (response.empty())
        {
            tcpManager->connected = false;
            return tcpManager;
        }
        
        tcpManager->messageReceived(response.at(0));
    }
    catch(...) {}
    
    tcpManager->connected = true;
    return tcpManager;
}

//ConnectionManager* newTCPConnectionManager()
